#ifndef __FLAGGER_COMPLEXMEDIANFLAGGER_H__
#define __FLAGGER_COMPLEXMEDIANFLAGGER_H__

#include <casa/Arrays.h>
#include <utility>
#include <vector>
#include <list>
#include <map>
#include "MS_File.h"

namespace WSRT
{
  using casa::Cube;
  using casa::Matrix;
  using casa::Complex;
  using casa::Int;
  using casa::Double;
  using casa::Bool;
  using casa::Table;
  using std::list;
  using std::vector;
  
  typedef pair<int, int> pairii;
  
  class ComplexMedianFlagger
  {
    public:
       ComplexMedianFlagger(MS_File* MSfile, 
                            int InputWindowSize, 
                            bool UseOnlyXpolarizations, 
                            double InputMinThreshold,
                            double InputMaxThreshold);
      ~ComplexMedianFlagger();

      void FlagDataOrBaselines(bool flagDatapoints, 
                               bool flagRMS,
                               bool ExistingFlags);
  
    protected:
      int                     NumAntennae;
      int                     NumPairs;
      int                     NumBands;
      int                     NumChannels;
      int                     NumPolarizations;
      int                     WindowSize;
      int                     NumTimeslots;
      double                  MinThreshold;
      double                  MaxThreshold;
      double                  MaxBaselineLength;
      double                  NoiseLevel;
      vector<double>          BaselineLengths;
      vector<pairii>          PairsIndex;
      map<pairii, int>        BaselineIndex;
      vector< Cube<Complex> > TimeslotData;
      vector< bool >          PolarizationsToCheck;
      vector<casa::String>    AntennaNames;
      Cube< int >             Statistics;
      MS_File* MSfile;
      
      void DeterminePolarizationsToCheck(Bool UseOnlyXpolarizations);
      void ComputeBaselineLengths();
      void ProcessStatistics();
      void FlagTimeslot(casa::TableIterator* flag_iter, 
                        bool flagDatapoints,
                        bool flagRMS,
                        bool ExistingFlags,
                        int Position);
      bool   FlagBaselineBand(Matrix<Bool>* Flags, 
                              Cube<Complex>* Timeslots, 
                              vector<double>* rms,
                              int* flagCounter,
                              double FlagThreshold,
                              int Position,
                              bool flagRMS);
      bool UpdateTimeslotData(vector<int>* OldFields,
                              vector<int>* OldBands,
                              int* TimeCounter,
                              Table* TimeslotTable,
                              double* Time);
    private:
  }; // ComplexMedianFlagger
}; // namespace WSRT

#endif //  __FLAGGER_COMPLEXMEDIANFLAGGER_H__
